/*

Fenryxo's library :-)

Copyright 2011 Jiří Janoušek <janousek.jiri@gmail.com>

If this file is built as a shared library it's released under GNU LGPL 3+ and 2-Clause BSD.
Otherwise it's released under GNU GPL 3+.
 
This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranties of 
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
PURPOSE.  See <http://www.gnu.org/licenses/> for more details.
*/

using Gee;
using Fenryxo.Logger;
namespace Fenryxo.Config{
	
	public abstract class ConfigDict: HashMap<string, string>{
		public bool readonly{get; set;}
		/**
		 * Creates new map representation of given file.
		 * 
		 * Creates new map representation of given file. Prints warning on parse errors.
		 * Note: key names are always lowercased.
		 * 
		 * @param file file to parse
		 */
		public ConfigDict(bool readonly=false){
			base();
			this.readonly = readonly;
		}
		/**
		 * Returns boolean value.
		 * 
		 * @param key option identifier
		 * @return false if value is empty or "off" (case insensitive), true otherwise
		 */
		public bool? @bool(string key, bool? @default=null){
			string? val = this[key];
			if(val == null) return @default;
			if(val == "on") return true;
			if(val == "off") return false;
			return @default;
		}
		
		/**
		 * Returns integer value.
		 * 
		 * @param key option identifier
		 * @return default value if value is empty or integer value
		 */
		public int? get_int(string key, int? @default=0){
			string? val = this[key];
			if(val == null) return @default;
			return int.parse(val);
		}
		
		/**
		 * Returns array of strings
		 * @param key option indentifier
		 * @return values
		 */
		public string[] array(string key){
			string[] result = {};
			var line = this[key];
			if(line == null) return result;
			
			string[] parts = line.split(",");
			if(parts.length == 0) return result;
			
			foreach(string s in parts){
				s = s.strip();
				if(s.length > 0) result += (owned) s;
			}
			parts = null;
			return result;
		}
		
		/**
		 * Saves configuration to file.
		 * 
		 * @throw Error on failure
		 */
		public abstract void save() throws Error;
		
		public signal void notify_changed();
	}
	
	/**
	 * Class parsing key-value configuration file.
	 * 
	 * Class parser configuration files which contain one key-value pair per line.
	 * Leading and traling white characters are removed.
	 * Key and value are separated by "=" (value may contain "=" character).
	 * Lines starting with a hash "#" are ignored.
	 * {{{
	 * foo = bar
	 * # nice comment
	 * apple = pear
	 * # This is also valid:
	 * comparation = <, =, >
	 * }}}
	 */
	public class ConfigFile: ConfigDict{
		private File file;
		/**
		 * Creates new map representation of given file.
		 * 
		 * Creates new map representation of given file. Prints warning on parse errors.
		 * Note: key names are always lowercased.
		 * 
		 * @param file file to parse
		 */
		public ConfigFile(File file, bool readonly=false){
			base(readonly);
			this.file = file;
			try{
				var dis = new DataInputStream(file.read());
				string line;
				int lineno = 0;
				while ((line = dis.read_line (null)) != null){
					lineno++;
					string item = line.strip();
					// Ignore empty lines or comments
					if(item.length == 0 || item[0] == '#') continue;
					string[] parts = item.split("=", 2);
					if(parts.length != 2){
						// Implicit keys without value are not supported
						lib_warning("ConfigFile: Parse error at %s:%i",
							file.get_uri(), lineno);
						continue;
					}
					string key = parts[0].strip();
					if(key.length == 0){
						// Empty key
						lib_warning("ConfigFile: Parse error at %s:%i",
							file.get_uri(), lineno);
						continue;
					}
					this[key] = parts[1].strip();
				}
			}
			catch(Error e){
				lib_debug("ConfigFile: Unable to read %s", file.get_uri());
			}
		}
		
		/**
		 * Saves configuration to file.
		 * 
		 * @throw Error on failure
		 */
		public override void save() throws Error{
			if(readonly) throw new IOError.READ_ONLY("Configuration file is opened read-only");
			var builder = new StringBuilder();
			foreach(var e in this.entries){
				builder.append(e.key);
				builder.append(" = ");
				builder.append(e.value);
				builder.append_c('\n');
			}
			string str = builder.str;
			try{
				file.get_parent().make_directory_with_parents();
			}
			catch(Error e){}
			file.replace_contents(str, str.length, null, false, FileCreateFlags.NONE, null);
		}
	}
}
