/*
Google Music Frame :: Application

This file contains main application class and its wrapper for DBus.

Copyright 2011 Jiří Janoušek <janousek.jiri@gmail.com>

This program is free software: you can redistribute it and/or modify it 
under the terms of the GNU General Public License version 3, as published 
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranties of 
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along 
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#if GLIB_DBUS

using Fenryxo.Logger;
using GMF;

namespace GMF.Helpers{
	
	/**
	 * Manages DockManager compliant docks
	 */
	public class DockManager: BaseHelper{
		private DBusDockManager dock;
		private DBusDockItem dock_item;
		private DockyDockItem docky_item;
		private Gee.Map<int32, Actions> dock_menu_items = null;
		private bool have_docky = false;
	
		
		/**
		 * Construct new manager
		 * 
		 * @param app Application object
		 * @param player player
		 */
		public DockManager(GMF.Application app, GMF.Player? player){
			base(app, player);
			this.dock_menu_items = new Gee.HashMap<int32, Actions>(null, null);
		}
		
		~ DockManager(){
			log_debug("~DockManager");
			this.remove_menu();
		}
		
		/**
		 * Starts to watch for docks
		 */
		public void start(){
			Bus.watch_name(BusType.SESSION, "net.launchpad.DockManager",
					BusNameWatcherFlags.NONE,
					this.dock_appeared, this.dock_vanished);
			
		}
		
		
		private void dock_appeared(DBusConnection conn, string name, string owner){
			log_debug("Dock appeared: %s, %s", name, owner);
			this.dock_item = null;
			try{
				this.dock = Bus.get_proxy_sync(BusType.SESSION,
				"net.launchpad.DockManager",
				"/net/launchpad/DockManager");
				
				/*
				 * Capabilities are used to detect Docky, because docky isn't fully
				 * compatible with DockManager specification (wrong signature
				 * of several methods).
				 */
				var cap = this.dock.get_capabilities();
				this.have_docky = ("x-docky-get-item-pids" in cap);
				if(this.have_docky){
					log_debug("DockManager: Using Docky compatibility mode.");
				}
				else{
					log_debug("DockManager: Using standard DockManager implementation.");
				}
				cap = null;
				this.dock.item_added.connect(this.check_dock_item);
				foreach(string path in this.dock.get_items()){
					this.check_dock_item(path);
					if(this.dock_item != null) break;
				}			
			}
			catch(IOError e){
				log_warning("Unable to get proxy for dock: %s", e.message);
				this.dock_cleanup();
				
			}
		}
		
		
		/**
		 * Check if the dock item is our dock item
		 * 
		 * @param path DBus path to dock item
		 */
		private void check_dock_item(string path){
			if(!this.have_docky){
				try{
					DBusDockItem item = Bus.get_proxy_sync(BusType.SESSION,
								"net.launchpad.DockManager", path);
					var desktop_entry = item.desktop_file;
					if(desktop_entry == null) return;
					log_debug("Dock item: %s", desktop_entry);
					if(desktop_entry.has_suffix(app.desktop_entry + ".desktop")){
						this.dock_item = item;
						this.docky_item = null;
						this.create_menu();
						item.menu_item_activated.connect(this.dock_menu_item_activated);
					}
				}
				catch(IOError e){
					log_warning("Dock.Manager: Unable to get DockItem proxy for %s: %s",
					            path, e.message);
				}
			}
			else{
				try{
					DockyDockItem d_item = Bus.get_proxy_sync(BusType.SESSION,
								"net.launchpad.DockManager", path);
					var desktop_entry = d_item.desktop_file;
					if(desktop_entry == null) return;
					log_debug("Dock item: %s", desktop_entry);
					if(desktop_entry.has_suffix(app.desktop_entry + ".desktop")){
						this.docky_item = d_item;
						this.dock_item = null;
						this.create_menu();
						d_item.menu_item_activated.connect(this.docky_menu_item_activated);
						/* Docky requires confirmation of menu items */
						d_item.menu_item_confirmation_needed.connect(
							this.docky_menu_item_confirmation_needed);
					}
				}
				catch(IOError e){
					log_warning("Dock.Manager: Unable to get DockItem proxy for %s: %s",
					            path, e.message);
				}
			}
		}
		
		protected override void create_menu(){
			
			Actions skip_action = this.player.playback_state == "playing"
				? Actions.PLAY : Actions.PAUSE;
			
			weak DBusDockItem item = null;
			weak DockyDockItem d_item = null;
			if(this.dock_item != null){
				item = this.dock_item;
			}
			else if(this.docky_item != null){
				d_item = this.docky_item;
			}
			else{
				return;
			}
			
			this.remove_menu();
			
			try{
				GLib.HashTable<string, Variant> menu_item;
//~ 				foreach(Actions action in Actions.all()){
				for(int a = 0; a < Actions.COUNT; a++){
					// Needed: If "a" is passed directly to this.trigger_action(),
					// it will be always 7 (the last iteration)
					Actions action = (Actions) a;
					if(action == skip_action) continue;
					menu_item = new GLib.HashTable<string, Variant>(null, null);
					menu_item.insert("label", action.get_label());
					menu_item.insert("icon-name", action.get_icon());
					if(item != null){
						this.dock_menu_items[item.add_menu_item(menu_item)] = action;
					}
					else{
						this.dock_menu_items[(int32) d_item.add_menu_item(menu_item)] = action;
					}
				}
			}
			catch(IOError e){
				this.remove_menu();
			}
			
		}
		
		protected override void update_menu(){
			this.create_menu();
		}
		
		
		/**
		 * Removes menu items from our dock item
		 */
		protected override void remove_menu(){
			if(this.dock_item != null){
				foreach(var id in this.dock_menu_items.keys){
					try{
						
						this.dock_item.remove_menu_item(id);
					}
					catch(IOError e){
						log_warning("App: Unable to remove dock menu item: %s", e.message);
					}
				}
			}
			else if(this.docky_item != null){
				foreach(var id in this.dock_menu_items.keys){
					try{
						
						this.docky_item.remove_menu_item((uint32) id);
					}
					catch(IOError e){
						log_warning("App: Unable to remove dock menu item: %s", e.message);
					}
				}
			}

			this.dock_menu_items.clear();
		}
		
		
		/**
		 * Confirms menu items at Docky
		 */
		private void docky_menu_item_confirmation_needed(){
			log_debug("Docky menu item confirmation");
			if(this.docky_item == null){
				log_warning("Docky menu item confirmation needed, but docky item is null.");
				return;
			}
//~ 			if(this.dock_menu_items == null || this.dock_menu_items.length == 0){
//~ 				log_warning("Docky menu item confirmation needed, but not menu items found.");
//~ 				return;
//~ 			}
			foreach(var id in this.dock_menu_items.keys){
				try{
					this.docky_item.confirm_menu_item((uint32) id);
				}
				catch(IOError e){
					log_warning("Unable to confirm docky menu item: %s", e.message);
				}
			}
		}
		
		/**
		 * Wrapper for DockManager.menu_item_activated signal for Docky due to
		 * signature incompatibility.
		 */
		private void docky_menu_item_activated(uint32 id){
			this.dock_menu_item_activated((int32) id);
		}
		
		/**
		 * Triggers menu item action
		 */
		private void dock_menu_item_activated(int32 id){
			log_debug(@"Dock menu item activated: $id");
			if(this.dock_menu_items.has_key(id)){
				this.trigger_action(this.dock_menu_items[id]);
			}
		}
		
		private void dock_vanished(DBusConnection conn, string name){
			log_debug("Dock vanished: %s", name);
			this.dock_cleanup();
		}
		
		private void dock_cleanup(){
			this.remove_menu();
			this.dock_item = null;
			this.docky_item = null;
			this.dock = null;
		}
	}
	
	[DBus(name = "net.launchpad.DockManager")]
	public interface DBusDockManager: Object{
		public abstract string[] get_capabilities() throws IOError;
		public abstract string[] get_items() throws IOError;
		public abstract string[] get_items_by_desktop_file(string desktop_file) throws IOError;
		public signal void item_added(string path);
		public signal void item_removed(string path);
		
	}
	
	[DBus(name = "net.launchpad.DockItem")]
	public interface DBusDockItem: Object{
		public abstract string desktop_file{ owned get;}
		public abstract int32 add_menu_item(GLib.HashTable<string, Variant> menu) throws IOError;
		public abstract void update_menu_item(GLib.HashTable<string, Variant> menu) throws IOError;
		public abstract void remove_menu_item(int32 id) throws IOError;
		public signal void menu_item_activated(int32 id);
	}
	
	[DBus(name = "net.launchpad.DockItem")]
	public interface DockyDockItem: Object{
		public abstract string desktop_file{ owned get;}
		public abstract uint32 add_menu_item(GLib.HashTable<string, Variant> menu) throws IOError;
		public abstract void update_menu_item(GLib.HashTable<string, Variant> menu) throws IOError;
		public abstract void confirm_menu_item(uint32 id) throws IOError;
		public abstract void remove_menu_item(uint32 id) throws IOError;
		public signal void menu_item_activated(uint32 id);
		public signal void menu_item_confirmation_needed();
	}
}
#endif
