/*
Google Music Frame :: Window

This file contains main window.

Copyright 2011 Jiří Janoušek <janousek.jiri@gmail.com>

This program is free software: you can redistribute it and/or modify it 
under the terms of the GNU General Public License version 3, as published 
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranties of 
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along 
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using Gtk;
using Fenryxo.Logger;
using Fenryxo.Path;
using Fenryxo.Widgets;

namespace GMF{
	public class MainWindow : Gtk.Window{
		private const string TITLE = "Google Music Frame";
		private Application app;
		private Toolbar bar;
		private Box box;
		
		public signal void go_home_action();
		public signal void refresh_action();
		public signal void troubleshooting_action();
		public signal void settings_action();
		public signal void close_action();
		
		/**
		 * Constructs new main window
		 * 
		 * @param app Application object
		 */
		public class MainWindow(Application app){
			this.app = app;
			this.position = WindowPosition.CENTER;
			this.title = TITLE;
			try {
				this.icon = IconTheme.get_default().load_icon(this.app.icon, 48, 0);
			}
			catch(Error e){
				log_warning("Unable to load application icon.");
			}
			
			this.bar = new Toolbar();
			this.bar.toolbar_style = ToolbarStyle.TEXT;
			ToolButton item;
			
			item = new ToolButton.from_stock(Stock.HOME);
			item.clicked.connect(() => { this.go_home_action(); });
			this.bar.add(item);
			
			item = new ToolButton.from_stock(Stock.REFRESH);
			item.clicked.connect(() => { this.refresh_action(); });
			this.bar.add(item);
			/// Main widow toolbar
			item = new ToolButton(null, _("Troubleshooting"));
			item.icon_name = Stock.PREFERENCES;
			item.clicked.connect(() => { this.troubleshooting_action(); });
			this.bar.add(item);
			
			item = new ToolButton.from_stock(Stock.PREFERENCES);
			item.clicked.connect(() => { this.settings_action(); });
			this.bar.add(item);
			
			item = new ToolButton.from_stock(Stock.ABOUT);
			item.clicked.connect(() => { this.show_about_dialog(); });
			this.bar.add(item);
			
			item = new ToolButton.from_stock(Stock.QUIT);
			item.clicked.connect(() => {
				try{
					this.app.quit();
				}
				catch(Fenryxo.AppError e){
					log_critical("Unable to quit: %s", e.message);
				}
			});
			this.bar.add(item);
			
			this.box = new Gtk.VBox(false, 2);
			this.box.pack_start(this.bar, false);
			base.add(this.box);
			this.box.show_all();
			
			
			/* Restore last window dimensions */
			int w = 0;
			int h = 0;
			var opt_w = this.app.ui_state["window_w"];
			if(opt_w != null) w = int.parse(opt_w);
			var opt_h = this.app.ui_state["window_h"];
			if(opt_h != null) h = int.parse(opt_h);
			this.set_default_size(w > 0 ? w: 980, h > 0 ? h : 600);
			
			/* restore position of window */
			var opt_x = this.app.ui_state["window_x"];
			var opt_y = this.app.ui_state["window_y"];
			if(opt_x != null && opt_y != null){
				this.move(int.parse(opt_x), int.parse(opt_y));
			}
			
			/* save UI state at exit */
			this.app.terminated.connect(this.on_quit);
		}
		
		public override void add(Widget w){
			this.box.pack_start(w);
		}
		
		public override void remove(Widget w){
			this.box.remove(w);
		}

#if FALSE		
		public override bool configure_event(Gdk.EventConfigure event){
			var result = base.configure_event(event);
			log_debug("UI state: %d, %d, %d, %d",
				event.x, event.y, event.width, event.height);
			this.app.ui_state["window_x"] = event.x.to_string();
			this.app.ui_state["window_y"] = event.y.to_string();
			this.app.ui_state["window_w"] = event.width.to_string();
			this.app.ui_state["window_h"] = event.height.to_string();
			this.app.ui_state.save();
			return result;
		}
#endif		
		/**
		 * Emmited when window is about to be closed.
		 * Default handler hides window, when state is "playing" and quit
		 * application otherwise
		 * 
		 * @param event delete event information
		 */
		public override bool delete_event(Gdk.Event event){
			this.hide();
			this.close_action();
			return true;
		}
		
		/**
		 * Save UI state and quit application
		 */
		private void on_quit(){
			this.app.ui_state["window_w"] = this.allocation.width.to_string();
			this.app.ui_state["window_h"] = this.allocation.height.to_string();
			try{
				this.app.ui_state.save();
				log_debug("Saved UI state: %s, %s",
					this.app.ui_state["window_w"], this.app.ui_state["window_h"]);
			}
			catch(Error e){
				log_warning("Unable to save UI state: %s", e.message);
			}
		}
		
		public void show_about_dialog(){
			AboutDialog.display(app);
			
		}
	}
	
	public class AboutDialog: Gtk.AboutDialog{
	
		private static AboutDialog win;
		
		public static void display(Application app){
			if(AboutDialog.win == null){
				AboutDialog.win = new AboutDialog(app);
			}
			AboutDialog.win.show_all();
			AboutDialog.win.present();
		}
		
		public AboutDialog(Application app){
			try {
				this.icon = IconTheme.get_default().load_icon(app.icon, 48, 0);
			}
			catch(Error e){
				log_warning("Unable to load application icon.");
			}
			this.copyright = GMF.Config.COPYRIGHT;
			this.website = GMF.Config.WEBSITE;
			this.version = GMF.Config.VERSION;
			this.program_name = GMF.Config.DISPLAY_NAME;
			this.logo_icon_name = app.icon;
			/// Shown in the About GMF dialog
			this.comments = _("Google Music integration for your Linux desktop");
			/// Names of translators (shown in the translator credits accessed from the About Google Music Frame dialog)
			this.translator_credits = _("translator-credits");
			this.artists = {"Alexander King <https://plus.google.com/117767008472331887156/about>"};
			this.authors = {"Jiří Janoušek <janousek.jiri@gmail.com>"};
			var content = this.get_content_area() as Gtk.Container;
			var table = new Gtk.Table(2, 2, true);
			content.add(table);

			
			var file = get_data_file(GMF.Config.APPNAME + "/images/info.png").get_path();
			/// Shown in the About GMF dialog, %s will be replaced by release version, e.g. 0.2
			var button = new ImageLinkButton((owned) file, GMF.Config.RELEASE_URL, _("About release %s").printf(GMF.Config.RELEASE));
			table.attach(button, 0, 1, 0, 1, 0, 0, 0, 0);


			file = get_data_file(GMF.Config.APPNAME + "/images/tip.png").get_path();			
			/// Shown in the About GMF dialog
			button = new ImageLinkButton((owned) file, GMF.Config.TIPS_URL, _("Tips"));
			table.attach(button, 1, 2, 0, 1, 0, 0, 0, 0);
			
			file = get_data_file(GMF.Config.APPNAME + "/images/donate.png").get_path();
			/// Shown in the About GMF dialog
			button = new ImageLinkButton((owned) file, GMF.Config.DONATE_URL, _("Donate"));
			table.attach(button, 2, 3, 0, 1, 0, 0, 0, 0);

			file = get_data_file(GMF.Config.APPNAME + "/images/bug.png").get_path();			
			/// Shown in the About GMF dialog
			button = new ImageLinkButton((owned) file, GMF.Config.BUG_TRACKER, _("Report a bug"));
			table.attach(button, 0, 1, 1, 2, 0, 0, 0, 0);
			
			file = get_data_file(GMF.Config.APPNAME + "/images/google-plus.png").get_path();
			/// Shown in the About GMF dialog
			button = new ImageLinkButton((owned) file, GMF.Config.GOOGLE_PLUS_URL, _("Google+ page"));
			table.attach(button, 1, 2, 1, 2, 0, 0, 0, 0);
			
			file = get_data_file(GMF.Config.APPNAME + "/images/facebook.png").get_path();
			/// Shown in the About GMF dialog
			button = new ImageLinkButton((owned) file, GMF.Config.FACEBOOK_URL, _("Facebook page"));
			table.attach(button, 2, 3, 1, 2, 0, 0, 0, 0);
			
		}
		
		public override bool delete_event(Gdk.Event event){
			AboutDialog.win = null;
			return false;
		}
		
		public override void response(int id){
			this.close();
		}
		
		public override void close(){
			AboutDialog.win = null;
			this.destroy();
		}
	}
}
